/**********************************************************BeginFile*************
* Copyright:
* (c) 2001 by CONSUL risk management b.v.  All Rights Reserved
*
* File-stamp: <020221 henk-jan 13:28 e2a.c>
*
* Name:
*   e2a.c
*
* Purpose: 
*   To convert an IFS file from EBCDIC to ASCII
*
* History: 
*   011128 0.0.2     HJvT   UE112013:  Created.
*   020214 0.0.10    HJvT   UE112013:  Replaced all CRMTrace() calls with
*                                      CRMWRITELOG(); added #include <localmsg.h>
*   020220 0.0.10    HJvT   UE112013:  Made sure that the \n is not converted to
*                                      a question mark
*   020221 0.0.10    HJvT   UE112013:  Debugged
***********************************************************EndFile**************/
#include <recio.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <ctype.h>

#include <baselib.h>
#include <crm.h>
#include <crmbase.h>
#include <utf8.h>
#include <localmsg.h>

/* Parameters */
#define DEFAULTBUFFERSIZE 32768

#define LOGDIR         "../log"
#define LOGFILENAME    LOGDIR"/e2a.log"

#define INFO_MESSAGE(msg)          CRMWRITELOG(CRM_AUD_DEVELOPER | CRM_AUD_OPERATOR | CRM_AUD_AUDITOR, \
                                               CRM_SVR_INFO,                                           \
                                               msg)
  
#define FATAL_ERROR_MESSAGE(msg)   CRMWRITELOG(CRM_AUD_DEVELOPER | CRM_AUD_OPERATOR | CRM_AUD_AUDITOR, \
                                               CRM_SVR_FATAL_ERROR,                                    \
                                               msg)

#define SEVERE_ERROR_MESSAGE(msg)  CRMWRITELOG(CRM_AUD_DEVELOPER | CRM_AUD_OPERATOR | CRM_AUD_AUDITOR, \
                                               CRM_SVR_SEVERE_ERROR,                                   \
                                               msg)

#define ERROR_MESSAGE(msg)         CRMWRITELOG(CRM_AUD_DEVELOPER | CRM_AUD_OPERATOR | CRM_AUD_AUDITOR, \
                                               CRM_SVR_ERROR,                                   \
                                               msg)

/* Global variables */
static Log *loghandle = NULL;

/* Prototypes of local functions */
char       *explainErrno();                 /* Explain the value of errno */


/**********************************************************BeginFunction********
* Name:
*   explainErrno - Explain the errno value
*
* History:
*   011122 0.0.1    HJvT   UE112013:  Created.
***********************************************************EndFunction*********/
char *explainErrno()
{
  char *result;
  char  errnoString[100];
  int   errnum;

  errnum = errno;
  errno  = 0;
  result = strerror(errnum);
  if (0 != errno)
    result = "<Unable to retrieve system message>";

  return result;
}


/**********************************************************BeginFunction********
* Name:
*   main - Main function
*
* Parameters: 
*   Name     Valid range  Usage
*   argv[1]  char[256]    Name of file to convert (IFS file path/name)
*   argv[2]  char[256]    Name to use for output file (IFS file path/name)
*
* History:
*   011128 0.0.2    HJvT   WE109005:  Created.
*   020220 0.0.10   HJvT   UE112013:  Made sure that the \n is not converted to
*                                     a question mark
*   020221 0.0.10   HJvT   UE112013:  Debugged
***********************************************************EndFunction*********/
int main(int   argc,
         char *argv[])
{
  int             error;
  void           *initparms[2]; /* For CRMLogOpen() */
  FILE           *inputFile;
  char            inputFileName[FILENAME_MAX + 1];
  char           *logFileName;
  FILE           *outputFile;
  char            outputFileName[FILENAME_MAX + 1];
  int             phase;

  phase = 1;
  error = 0;

  if (0 == error)
  {
    phase++;

    logFileName = LOGFILENAME;

    if (CRM_RC_OK != CRMLogInit())
    {
      fprintf(stderr, "%s: Failed to initiate logging\n",
              argv[0]);
      error = phase;
    }
  }
    
  if (0 == error)
  {
    phase++;

    initparms[0] = (void *) logFileName;
    initparms[1] = (void *) 2000;

    if (CRMLogOpen(&loghandle,
                   logFileName,
                   2,
                   initparms,
                   OpenBinLog,
                   WriteToBinLog,
                   CloseBinLog) != CRM_RC_OK)
    {
      fprintf(stderr, "%s: Failed to open log file: %s\n",
              argv[0],
              logFileName);
      error = phase;
    }
  }

  if (0 == error)
  {
    phase++;

    if (3 != argc)
    {
      error = phase;

      FATAL_ERROR_MESSAGE(LOG_OS400_E2A_USAGE(argv[0]));
    }
  }
    
  if (0 == error)
  {
    phase++;
    
    strncpy(inputFileName,  argv[1], FILENAME_MAX);
    strncpy(outputFileName, argv[2], FILENAME_MAX);
    inputFileName[FILENAME_MAX]  = '\0';
    outputFileName[FILENAME_MAX] = '\0';

    INFO_MESSAGE(LOG_OS400_INPUT_AND_OUTPUT_FILE(argv[0],
                                                 inputFileName,
                                                 outputFileName));
  }
  
  /* Open the input file */
  if (0 == error)
  {
    phase++;
    
    CRMTrace("OS400: %s: Opening input file <%s>",
             argv[0],
             inputFileName);

    inputFile = fopen(inputFileName, "r");
    if (NULL == inputFile)
    {
      error = phase;

      FATAL_ERROR_MESSAGE(LOG_OS400_CANNOT_OPEN_INPUT_FILE(argv[0],
                                                           inputFileName,
                                                           explainErrno()));
    }
  }
    
  /* Open the output file */
  if (0 == error)
  {
    phase++;
    
    CRMTrace("OS400: %s: Opening output file <%s>",
             argv[0],
             outputFileName);
    outputFile = fopen(outputFileName, "w");
    if (outputFile == NULL)
    {
      error = phase;
      
      FATAL_ERROR_MESSAGE(LOG_OS400_CANNOT_OPEN_OUTPUT_FILE(argv[0],
                                                            outputFileName,
                                                            explainErrno()));
    }
  }
  
  /* Convert the input file */
  if (0 == error)
  {
    int        index;
    size_t     numberOfBytesRead;
    CRMBYTE    outputLine[DEFAULTBUFFERSIZE];
    char       record[DEFAULTBUFFERSIZE];
    CRMUINT32  size;
    CRMBOOLEAN endOfFile;
      
    phase++;
      
    /*
      Read all lines of the input file,
      convert them to ASCII
      and write them in the output file
    */
    endOfFile = 0;
    while ((0 == endOfFile) && (0 == error))
    {
      errno = 0;
      
      numberOfBytesRead = fread(record, 1, sizeof(record), inputFile);
      if (0 != errno)
      {
        error = phase;
        
        FATAL_ERROR_MESSAGE(LOG_OS400_CANNOT_READ_INPUT_FILE(argv[0],
                                                             inputFileName,
                                                             explainErrno()));
      }

      if (0 == error)
      {
        if (0 != feof(inputFile))
          endOfFile = 1;

        /*
          Convert the line to ASCII (UTF8?) and write it in the output file
        */
        /* Convert non-printable characters to question marks */
        /* (\n is not printable, but it is convertible) */
        for (index = 0; 
             index < numberOfBytesRead; 
             index++)
        {
          if (0 == isprint(record[index])
              && '\n' != record[index])
            record[index] = '?';
        }

        /* Convert to UTF8 */
        if (0 != ASC8Encode(record,
                            numberOfBytesRead,
                            outputLine,
                            &size))
        {
          ERROR_MESSAGE(LOG_OS400_CANNOT_CONVERT_RECORD(argv[0],
                                                        record));
          /* error = phase; // We don't want to abort here */
        }

        if (size >= sizeof(outputLine))
        {
          ERROR_MESSAGE(LOG_OS400_RECORD_TOO_LONG(argv[0],
                                                  record));
          outputLine[sizeof(outputLine)] = '\0';
          /* error = phase; // We don't want to abort here */
        }
      }

      /* Write to the output file */
      if (0 == error)
      {
        /* Append a null character */
        if (size < sizeof(outputLine))
          outputLine[size] = '\0';

        errno = 0;
        fprintf(outputFile,
                "%s",
                outputLine);
        switch (errno)
        {
          case 0:
            /* All is well (?) */
            break;
      
          case ENOSPC:
            error = phase;

            FATAL_ERROR_MESSAGE(LOG_OS400_DISK_FULL(argv[0],
                                                    outputFileName));
            break;
            
          default:
            error = phase;

            FATAL_ERROR_MESSAGE(LOG_OS400_CANNOT_WRITE_OUTPUT_FILE(argv[0],
                                                                   outputFileName,
                                                                   explainErrno()));
        }
      }      
    } /* End while */
  } 
    
  
  if (NULL != loghandle)
  {
    INFO_MESSAGE(LOG_OS400_FINISHED(argv[0]));

    CRMLogClose(loghandle,
                0,
                NULL);
  }
  
  CRMLogExit();
  
  return error;
}
